@@ -1,5 +1,13 @@
+import json
 from aiohttp import FormData
+from aiohttp import web
+from aiohttp.test_utils import TestClient
 import pytest
+
+from rmlab_http_client import (
+    SyncClient,
+)
+
 from rmlab_http_client.types import (
     AuthType,
     DataRequestContext,
@@ -12,7 +20,7 @@ from rmlab_http_client.types import (
     ResponseType,
     DataRequestContext,
 )
-from rmlab_errors import ValueError
+from rmlab_errors import ValueError, error_handler, exception_to_http_code
 
 
 async def resource_vars(*, foo: str, bar: int = 32, zee: float, boo: bool):
@@ -226,3 +234,95 @@ def test_files():
             ep, foo="foovalue", bar=15, file_content="not_existing"
         ):
             pass
+
+
+async def resource_echo_dict(*, dict_arg: dict):
+
+    return dict_arg
+
+
+async def server_generic(request: web.Request):
+
+    try:
+        data = await request.json()
+
+        response = await resource_echo_dict(**data)
+
+        resp_payload = {
+            "status": 200,
+            "content_type": "application/json",
+            "body": json.dumps(response).encode(),
+        }
+
+    except Exception as exc:
+
+        resp_payload = {
+            "status": exception_to_http_code(exc),
+            "content_type": "application/json",
+            "body": json.dumps(error_handler(exc)).encode(),
+        }
+
+    finally:
+
+        return web.Response(**resp_payload)
+
+
+async def test_dict_types(aiohttp_client):
+
+    pargs = PayloadArguments.make_from_function(resource_echo_dict)
+
+    ep = Endpoint(
+        resource=["my", "resource"],
+        method=MethodType.GET,
+        payload=PayloadType.JSON,
+        auth=AuthType.PUBLIC,
+        response=ResponseType.JSON,
+        arguments=pargs,
+    )
+    request_data = DataRequestContext.make_data(
+        ep,
+        dict_arg={
+            "one": 1,
+            1: "one",
+            "null": None,
+            "bool_true": True,
+            "bool_false": False,
+        },
+    )
+
+    # All keys and values types in dict are preserved when making DataRequestContext
+    assert "one" in request_data["dict_arg"]
+    assert request_data["dict_arg"]["one"] == 1
+    assert 1 in request_data["dict_arg"]
+    assert request_data["dict_arg"][1] == "one"
+    assert "null" in request_data["dict_arg"]
+    assert request_data["dict_arg"]["null"] == None
+    assert "bool_true" in request_data["dict_arg"]
+    assert request_data["dict_arg"]["bool_true"] == True
+    assert "bool_false" in request_data["dict_arg"]
+    assert request_data["dict_arg"]["bool_false"] == False
+
+    app = web.Application()
+    app.router.add_route("get", ep.address, server_generic)
+    client: TestClient = await aiohttp_client(app)
+
+    async with SyncClient(
+        ep,
+        address="http://" + client.host + ":" + str(client.port),
+    ) as http_client:
+
+        resp_data = await http_client.submit_request(request_data)
+
+    # ... but integer keys are received as strings in the server, and it is fine
+    # Integer values are received as integers in the server
+
+    assert "one" in resp_data
+    assert resp_data["one"] == 1
+    assert "1" in resp_data
+    assert resp_data["1"] == "one"
+    assert "null" in resp_data
+    assert resp_data["null"] == None
+    assert "bool_true" in resp_data
+    assert resp_data["bool_true"] == True
+    assert "bool_false" in resp_data
+    assert resp_data["bool_false"] == False
